/*
*	Race: Minotavr Functions
*/

new g_MN_HealingAura[33];

MN_ULT_Respawn( idUser )
{
	
	// User can't use ultimate while planting or defusing!
	if ( p_data_b[idUser][PB_ISPLANTING] )
{
		client_print( idUser, print_center, "%L",LANG_PLAYER,"CLIENT_PRINT_WA_ULT_VENGEANCE");
				
		return;
	}

	// If the user's ultimate is not available, then why are we here?
	if ( !ULT_Available( idUser ) )
	{		
		return;
	}

	// We need to respawn the user!!
	if ( !is_user_alive( idUser ) )
	{

		// Make sure the user isn't changing teams and we're not at the end of the round
		if ( !p_data[idUser][P_RESPAWNBY] )
		{

			// Remove the ultimate delay if it existed (in theory it shouldn't)
			if ( task_exists( TASK_UDELAY + idUser ) )
			{
				remove_task( TASK_UDELAY + idUser );
			}

			// Set up a new ultimate delay
			ULT_ResetCooldown( idUser, get_pcvar_num( CVAR_wc3_ult_cooldown ) );

			// Set up the respawn task
			set_task( SPAWN_DELAY, "_SHARED_Spawn", TASK_SPAWN + idUser );

			// We need to set that we're about to respawn
			p_data[idUser][P_RESPAWNBY] = RESPAWN_MINOTAVR;
		}
	}
	
	// Otherwise teleport back to spawn
	else
	{		
		// Lets find a free spawn and obey immunity rules
		new ent = SHARED_FindFreeSpawn( idUser, true );
		
		// No spawn found
		if ( ent < 0 )
		{
			//set_hudmessage( 255, 255, 10, -1.0, -0.4, 1, 0.5, 1.5, 0.2, 0.2 ,5 );
			
			// Immune person nearby
			if ( ent == -2 )
			{
				// The user's ultimate was blocked!
				ULT_Blocked( idUser );

				// Set up a new ultimate delay
				ULT_ResetCooldown( idUser, get_pcvar_num( CVAR_wc3_ult_cooldown ) );

				WC3_StatusText( idUser, 0, "%L", LANG_PLAYER, "VENGEANCE_FAILED_ENEMY_IMMUNITY" );
			}

			// No spawn found
			else
			{
				WC3_StatusText( idUser, 0, "%L", LANG_PLAYER, "NO_FREE_SPAWN_FOUND" );
			}
		}

		// Spawn found!
		else
		{
			new vOrigin[3], vNewOrigin[3], Float:fSpawnOrigin[3];
			
			// Get the user's origin
			get_user_origin( idUser, vOrigin );

			// Get the origin of the spawn
			entity_get_vector( ent, EV_VEC_origin, fSpawnOrigin );
			
			// Convert float vector to int vector
			FVecIVec( fSpawnOrigin, vNewOrigin );
			
			// Move the user
			set_user_origin( idUser, vNewOrigin );

			// Play teleport animation at old location
			Create_TE_TELEPORT( vOrigin );

			// Change user's health
			set_user_health( idUser, hp_ult_Minotavr );

			// Play the Vengeance sound		
			emit_sound( idUser, CHAN_STATIC, g_szSounds[SOUND_VENGEANCE], 1.0, ATTN_NORM, 0, PITCH_NORM );

			// Set up a new ultimate delay
			ULT_ResetCooldown( idUser, get_pcvar_num( CVAR_wc3_ult_cooldown ) );
		}
	}

	return;
}

MN_HealingAura( idUser )
{
	if ( !is_user_alive( idUser ) )
	{
		return;
	}

	static iSkillLevel, HealingAuraGiven, iHealth;
 	iSkillLevel = SM_GetSkillLevel( idUser, SKILL_MINOTAVR3 );
 	
	// Already given
	HealingAuraGiven = g_MN_HealingAura[idUser];
		
	// Then the user has devotion aura
	if ( iSkillLevel > 0 )
	{
		g_MN_HealingAura[idUser] = p_minotavr3 * iSkillLevel;
	}
	else
	{
		g_MN_HealingAura[idUser] = 0;
	}
 
	// Player may even lose HP because of this
	HealingAuraGiven = g_MN_HealingAura[idUser] - HealingAuraGiven;

	iHealth = get_user_health( idUser );
	if (iHealth + HealingAuraGiven < 0)
	{
		set_user_health( idUser, 1 );
 	}

	if (iHealth + HealingAuraGiven > 255)
	{
		set_user_health( idUser, 255 );
	}
	else
	{
			set_user_health( idUser, iHealth + HealingAuraGiven );
	}

}


// Helper function to start the healing process
public MN_SkillRegen( id )
{
	if ( task_exists( id + TASK_MINOTAVR_SKILL ) == 0 )
	{
		_MN_SkillRegen( id );
	}
}

// We don't need to ever actually "turn this off" since this task will take care of itself
public _MN_SkillRegen( id )
{
	
	if ( id >= TASK_MINOTAVR_SKILL )
	{
		id -= TASK_MINOTAVR_SKILL;
	}

	// User is no longer connected :/
	if ( !p_data_b[id][PB_ISCONNECTED] )
	{
		return;
	}
	
	static iSkillLevel;
	iSkillLevel = SM_GetSkillLevel( id, SKILL_MINOTAVR2 );

	// User doesn't have this skill, so lets return
	//  - Also return if this was called twice
	if ( iSkillLevel > 0 && p_data_b[id][PB_ISFROZENSKILL] == false )
	{

	// Continue healing...
	if ( is_user_alive( id ) )
	{
		set_task( p_minotavr2[iSkillLevel-1], "_MN_SkillRegen", TASK_MINOTAVR_SKILL + id );
	}
	
	// Prevent healing if this player is hexed
	if ( p_data_b[id][PB_HEXED] )
	{
		return;
	}


	new players[32], numberofplayers, targetid, targetorigin[3], origin[3];
	new iTeam = get_user_team( id );

	get_user_origin( id, origin );
	get_players( players, numberofplayers, "a" );
	
	// Loop through each player and check to see if one is close enough to be healed
	for ( new i = 0; i < numberofplayers; i++ )
	{
		targetid = players[i];
		
		// User is on the caster's team
		if ( p_data_b[targetid][PB_ISCONNECTED] && get_user_team( targetid ) == iTeam )
		{
			get_user_origin( targetid, targetorigin );
			
			// User is close enough
			if ( get_distance(origin, targetorigin) < hw_range_Minotavr )
			{
				get_user_origin( targetid, origin )
				
				// User needs health
				if ( get_user_health( targetid ) + b_h_Minotavr <= get_user_maxhealth( targetid ) )
				{
					if (get_user_health( targetid ) + b_h_Minotavr > 315)
					{
						set_user_health( targetid, 315 );
					}
					else
					{
						set_user_health( targetid, get_user_health( targetid ) + b_h_Minotavr );
						Create_TE_IMPLOSION( origin, 100, 8, 1 );
					}
				}
			}
		}
	}
	}

	return;
}

MN_EvadeSkill( id, iHitZone )
{
	static iSkillLevel;

	// Check to see if they should evade this shot?
	iSkillLevel = SM_GetSkillLevel( id, SKILL_MINOTAVR1 );

	if ( !p_data_b[id][PB_HEXED] && iSkillLevel > 0 && random_float( 0.0, 1.0 ) <= p_minotavr1[iSkillLevel-1] && p_data_b[id][PB_ISFROZENSKILL] == false )
	{
		new iGlowIntensity = random_num( 20, 50 );
		
		// Head shot
		if ( iHitZone & (1 << HITGROUP_HEAD) )
		{
			iGlowIntensity += 250;
		}

		// Chest
		else if ( iHitZone & (1 << HITGROUP_CHEST) )
		{
			iGlowIntensity += 75;
		}
		
		// Make the user glow!
		SHARED_Glow( id, 0, 0, iGlowIntensity, 0 );

		Create_ScreenFade( id, (1<<10), (1<<10), (1<<12), 0, 0, 255, g_GlowLevel[id][2] );

		return 1;
	}

	return 0;
}
