/*
*	Race: Undead Scourge Functions
*/

UD_Suicide( idUser )
{
	
	// Ultimate has been used, so we can't use it again!
	if ( !ULT_Available( idUser ) )
	{
		return;
	}

	// If we ult while changing teams, you will kill your old teammates, we don't want that!!
	if ( p_data_b[idUser][PB_CHANGINGTEAM] )
	{
		return;
	}

	// Play the undead explosion sound!!!
	emit_sound( idUser, CHAN_STATIC, g_szSounds[SOUND_SUICIDE], 1.0, ATTN_NORM, 0, PITCH_NORM );

	new parm[5], vOrigin[3];
	get_user_origin( idUser, vOrigin );

	parm[0] = idUser;
	parm[1] = 6;
	parm[2] = vOrigin[0];
	parm[3] = vOrigin[1];
	parm[4] = vOrigin[2];

	// Set up the tasks to damage + draw effects
	set_task( 0.5, "_UD_SuicideExplode", TASK_EXPLOSION + idUser, parm, 5 );
	set_task( 0.5, "_UD_SuicideBlastCircles", TASK_BEAMCYLINDER + idUser, parm, 5 );
	
	// Create an implosion effect where they died
	Create_TE_IMPLOSION( vOrigin, 100, 20, 5 );
	
	// Set up an ultimate delay (in case the user respawns)
	ULT_ResetCooldown( idUser, get_pcvar_num( CVAR_wc3_ult_cooldown ) );
}


// This is only use in _UD_SuicideExplode so declaring it here should be fine
new bool:bIgnoreDmg[33] = false;

// Draw the explosions
public _UD_SuicideExplode( parm[5] )
{
	new idUser = parm[0];

	if ( get_pcvar_num( CVAR_wc3_psychostats ) )
	{
		new WEAPON = CSW_SUICIDE - CSW_WAR3_MIN;

		iStatsShots[idUser][WEAPON]++;
	}

	new vOrigin[3], vPosition[3];
	vOrigin[0] = parm[2];
	vOrigin[1] = parm[3];
	vOrigin[2] = parm[4];

	vPosition[0] = vOrigin[0] + random_num( -100, 100 );
	vPosition[1] = vOrigin[1] + random_num( -100, 100 );
	vPosition[2] = vOrigin[2] + random_num( -50, 50 );
	
	Create_TE_EXPLOSION( vOrigin, vPosition, g_iSprites[SPR_FIREBALL], (random_num(0,20) + 20), 12, TE_EXPLFLAG_NONE );

	// This doesn't look correct in Day of Defeat, so lets only do it for CS/CZ
	if ( g_MOD == GAME_CSTRIKE || g_MOD == GAME_CZERO )
	{
		Create_TE_Smoke( vOrigin, vPosition, g_iSprites[SPR_SMOKE], 60, 10 );
	}

	new players[32], numberofplayers;
	get_players( players, numberofplayers, "a" );

	new i, iTargetID, vTargetOrigin[3], iDamage, iDistance;
	new iMultiplier = ( explo_max_dmg_undead * explo_max_dmg_undead ) / explo_range_undead;
	new iTeam = get_user_team( idUser );
	
	// Check all players and see if they should be damaged
	for ( i = 0; i < numberofplayers; i++ )
	{
		iTargetID = players[i];
		
		// Get origin of target
		get_user_origin( iTargetID, vTargetOrigin );

		// Get distance in b/t target and iAttacker
		iDistance = get_distance( vOrigin, vTargetOrigin );
		
		// Make sure this user is close enough to do damage + isn't immune + isn't on the same team + isn't already immune to all damage
		if ( iDistance < explo_range_undead && iTeam != get_user_team( iTargetID ) && !bIgnoreDmg[iTargetID] )
		{
			static iSkillLevel2;
			iSkillLevel2 = SM_GetSkillLevel( iTargetID, PASS_DEATH_KNITH );
			if ( iSkillLevel2 > 0 && p_data_b[iTargetID][PB_ISFROZENSKILL] == false &&
			!( ITEM_Has( iTargetID, ITEM_NECKLACE ) > ITEM_NONE || (fWorkTalismanPassive(iTargetID, arrTalismanActive[iTargetID]) == TALISMAN_ITEM_NECKLACE && arrTalismanNecklaceNum[iTargetID] > 0)))
			{
			if ( random_float(0.0, 1.0 ) <= p_undead_rab[p_data[iTargetID][P_LEVEL]] )		
			{
			ULT_Blocked( idUser );
			bIgnoreDmg[iTargetID] = true;
			}
			else
			{
			// Calculate the damage to be done
				iDamage = ( explo_range_undead - iDistance) * iMultiplier;
				iDamage = sqroot( iDamage );
				
				// Damage them!!!!
				WC3_Damage( iTargetID, idUser, iDamage, CSW_SUICIDE, -1 );
				
				// Lets shake up their screen a bit
				Create_ScreenShake( iTargetID, (1<<14), (1<<13), (1<<14) );
			}
			}
			else
			{
			if ( ULT_CanUserBlockUlt( iTargetID ) )
			{
				ULT_RemoveCharge( iTargetID );

				ULT_Blocked( idUser );
				
				bIgnoreDmg[iTargetID] = true;
			}
			else
			{
			
				// Calculate the damage to be done
				iDamage = ( explo_range_undead - iDistance) * iMultiplier;
				iDamage = sqroot( iDamage );
				
				// Damage them!!!!
				WC3_Damage( iTargetID, idUser, iDamage, CSW_SUICIDE, -1 );
				
				// Lets shake up their screen a bit
				Create_ScreenShake( iTargetID, (1<<14), (1<<13), (1<<14) );
			}
			}
		}

		// Reset the "don't damage" rule
		if ( parm[1] - 1 <= 0 )
		{
			bIgnoreDmg[iTargetID] = false;
		}
	}

	--parm[1];
	
	// Lets keep going if we have some left!
	if ( parm[1] > 0 )
	{
		set_task( 0.1, "_UD_SuicideExplode", TASK_EXPLOSION + idUser, parm, 5 );
	}
}

// Draw the blast circles
public _UD_SuicideBlastCircles( parm[5] )
{
	new vOrigin[3], vPosition[3];

	vOrigin[0] = parm[2];
	vOrigin[1] = parm[3];
	vOrigin[2] = parm[4] - 16;

	vPosition[0] = vOrigin[0];
	vPosition[1] = vOrigin[1];
	vPosition[2] = vOrigin[2] + explo_blast_undead;

	Create_TE_BEAMCYLINDER( vOrigin, vOrigin, vPosition, g_iSprites[SPR_SHOCKWAVE], 0, 0, 6, 16, 0, 188, 220, 255, 255, 0 );

	vOrigin[2] = ( vOrigin[2] - explo_blast_undead ) + ( explo_blast_undead / 2 );

	Create_TE_BEAMCYLINDER( vOrigin, vOrigin, vPosition, g_iSprites[SPR_SHOCKWAVE], 0, 0, 6, 16, 0, 188, 220, 255, 255, 0 );
}


UD_SkillsOffensive( iAttacker,iVictim, iDamage )
{
	static iSkillLevel;

	if ( !p_data_b[iAttacker][PB_ISCONNECTED] )
	{
		return;
	}

	// Vampiric Aura
	iSkillLevel = SM_GetSkillLevel( iAttacker, SKILL_VAMPIRICAURA );
	if ( iSkillLevel > 0 && p_data_b[iAttacker][PB_ISCONNECTED] && p_data_b[iAttacker][PB_ISFROZENSKILL] == false)
	{
		static iHealth, iMaxHealth, iMaxHealthvictim, iHealthvictim, iBonusHealth, iBonusHealth2;

		static iSkillLevel2;
		iSkillLevel2 = SM_GetSkillLevel( iVictim, PASS_DEATH_KNITH );
		if ( iSkillLevel2 > 0 && p_data_b[iVictim][PB_ISFROZENSKILL] == false )
		{
		if ( random_float(0.0, 1.0 ) <= p_undead_rab[p_data[iVictim][P_LEVEL]] )		
		{
		iHealth		= get_user_health( iAttacker );
		iMaxHealth	= get_user_maxhealth( iAttacker );
		iMaxHealthvictim = get_user_maxhealth( iVictim );
		iHealthvictim = get_user_health( iVictim );
		iBonusHealth2 = floatround( float( iDamage ) * p_vampiric[iSkillLevel-1] * php_give_DKdf);
		iBonusHealth = floatround( float( iDamage ) * p_vampiric[iSkillLevel-1] * (1.0 - php_give_DKdf));
		
		// Give the user health!
		if ( get_user_health( iVictim ) < get_user_maxhealth( iVictim ) )
		{

			// Then give the user his maximum health
			if ( iHealthvictim + iBonusHealth2 > iMaxHealthvictim )
			{
				set_user_health( iVictim, iMaxHealthvictim );
			}

			// Otherwise just give iMaxHealth
			else
			{
				set_user_health( iVictim, iHealthvictim + iBonusHealth2 );
			}
		}
		
		if ( iHealth < iMaxHealth )
		{

			// Then give the user his maximum health
			if ( iHealth + iBonusHealth > iMaxHealth )
			{
				set_user_health( iAttacker, iMaxHealth );
			}

			// Otherwise just give iMaxHealth
			else
			{
				set_user_health( iAttacker, iHealth + iBonusHealth );
			}
		}
		
		// Make the attacker glow
		//SHARED_Glow( iAttacker, 0, ( 2 * iBonusHealth ), 0, 0 );
		SHARED_GlowShell(iVictim,0,238,118,get_pcvar_float( CVAR_wc3_glowshell_time ),get_pcvar_num( CVAR_wc3_glowshell_amount ));
		
		// Give the attacker a nice screen fade
		Create_ScreenFade( iAttacker, (1<<10), (1<<10), (1<<12), 0, 255, 0, iBonusHealth );
		}
		else
		{
		iHealth		= get_user_health( iAttacker );
		iMaxHealth	= get_user_maxhealth( iAttacker );
		iBonusHealth = floatround( float( iDamage ) * p_vampiric[iSkillLevel-1] );
		
		// Give the user health!
		if ( iHealth < iMaxHealth )
		{

			// Then give the user his maximum health
			if ( iHealth + iBonusHealth > iMaxHealth )
			{
				set_user_health( iAttacker, iMaxHealth );
			}

			// Otherwise just give iMaxHealth
			else
			{
				set_user_health( iAttacker, iHealth + iBonusHealth );
			}
		}
		
		// Make the attacker glow
		//SHARED_Glow( iAttacker, 0, ( 2 * iBonusHealth ), 0, 0 );
		SHARED_GlowShell(iVictim,0,238,118,get_pcvar_float( CVAR_wc3_glowshell_time ),get_pcvar_num( CVAR_wc3_glowshell_amount ));
		
		// Give the attacker a nice screen fade
		Create_ScreenFade( iAttacker, (1<<10), (1<<10), (1<<12), 0, 255, 0, iBonusHealth );
		}
		}
		else
		{
		
		
		iHealth		= get_user_health( iAttacker );
		iMaxHealth	= get_user_maxhealth( iAttacker );
		iBonusHealth = floatround( float( iDamage ) * p_vampiric[iSkillLevel-1] );
		
		// Give the user health!
		if ( iHealth < iMaxHealth )
		{

			// Then give the user his maximum health
			if ( iHealth + iBonusHealth > iMaxHealth )
			{
				set_user_health( iAttacker, iMaxHealth );
			}

			// Otherwise just give iMaxHealth
			else
			{
				set_user_health( iAttacker, iHealth + iBonusHealth );
			}
		}
		
		// Make the attacker glow
		//SHARED_Glow( iAttacker, 0, ( 2 * iBonusHealth ), 0, 0 );
		SHARED_GlowShell(iVictim,0,238,118,get_pcvar_float( CVAR_wc3_glowshell_time ),get_pcvar_num( CVAR_wc3_glowshell_amount ));
		
		// Give the attacker a nice screen fade
		Create_ScreenFade( iAttacker, (1<<10), (1<<10), (1<<12), 0, 255, 0, iBonusHealth );
		}
	}
}
